<?php
/**
 * @version		$Id: similarity_base.class.php,v 1.9 2009/09/14 17:04:13 juanca Exp $
 * @package		vpl. Similarity base and utility clases
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

require_once('tokenizer_factory.class.php');

/**
 * @abstract class to represent files from any source 
 */
class vpl_file_from_base{
	public function show_info(){
	}
}

/**
 * Represent information of a file from a directory
 */
class vpl_file_from_dir extends vpl_file_from_base{
	static $usersname=array();
	var $filename;
	var $userid;
	static public function add_user_name($userid,$username){
		if(!isset(self::$usersname[$userid])){
			self::$usersname[$userid]=$username;
		}
	}
	function __construct(&$filename,$userid=''){
		$this->filename=$filename;
		$this->userid=$userid;
	}
	public function show_info(){
		$ret='';
		$ret .='<a href="'.'file.php'.'">';
		$ret .=s($this->filename).' ';
		$ret .= '</a>';
		if($this->userid!=''){
			$ret .= ' '.self::$usersname[$this->userid];
		}
		return $ret;		
	}
}

/**
 * Represent information of a file from a zip file
 *
 */
class vpl_file_from_zipfile extends vpl_file_from_dir{
	function __construct(&$filename,$userid=''){
		parent::__construct($filename,$userid);
	}
	public function show_info(){
		$ret='';
		$ret .=s($this->filename);
		if($this->userid!=''){
			$ret .= ' '.self::$usersname[$this->userid];
		}
		return $ret;
	}
}

/**
 * Represent information of a file from other vpl activity
 *
 */
class vpl_file_from_activity extends vpl_file_from_base{
	static $vpls=array();
	var $vplid;
	var $filename;
	var $userid;
	function __construct(&$filename,&$vpl,$userid=-1){
		$id = $vpl->get_instance()->id;
		if(!isset(self::$vpls[$id])){
			self::$vpls[$id]=$vpl;
		}
		$this->vplid = $id;
		$this->filename=$filename;
		$this->userid=$userid;
	}
	public function show_info(){
		$vpl = self::$vpls[$this->vplid];
		$cmid = $vpl->get_course_module()->id;
		$ret='';
		if($this->userid>=0){
			$user = get_record('user','id',$this->userid);
		}else{
			$user = false;
		}
		if($user){
			$ret .='<a href="'.vpl_mod_href('/forms/submissionview.php','id',$cmid,'userid',$user->id).'">';
		}
		$ret .=s($this->filename).' ';
		if($user){
			$ret .= '</a>';
			$ret .= print_user_picture($user,$vpl->get_course()->id,TRUE,FALSE,TRUE).' '.fullname($user);
		}
		return $ret;		
	}
}


/**
 * Represent similarity preprocesing information of a file
 * from any source (directory, zip file or vpl activity)
 *
 */
class vpl_similarity_base{
	var $from;
	var $size;
	var $vector;
	public function get_type(){
		return 0;
	}
	public function get_tokenizer(){
		return vpl_tokenizer_factory::get('base');
	}
	public function sintax_normalize(&$tokens){
		return $tokens;
	}
	public function init(&$data,&$from){
		$this->from=$from;
		$this->size=0;
		$this->vector = array();
		$tok = $this->get_tokenizer();
		$tok->parse($data);
		$tokens = $this->sintax_normalize($tok->get_tokens());
		foreach($tokens as $token){
			if($token->type == vpl_token_type::operator){
				$this->size++;
				if(isset($this->vector[$token->value])){
					$this->vector[$token->value]++;
				}else{
					$this->vector[$token->value]=1;
				}
			}
		}
	}
	public function show_info(){
		return $this->from->show_info();
	}
	public function similarity1(&$other){
		$dif1=0;
		$dif2=0;
		foreach($this->vector as $op => $frec){
			if(isset($other->vector[$op])){
				if($frec != $other->vector[$op]){
					$dif1++;
				}
			}else{
				$dif2++;
			}
		}
		$dif2 += (count($other->vector))-(count($this->vector)-$dif2);
		return 100*(1-(($dif1+$dif2)/(count($this->vector)+count($other->vector))));
	}

	public function similarity2(&$other){
		$dif=0;
		$taken=0;
		foreach($this->vector as $op => $frec){
			if(isset($other->vector[$op])){
				$dif += abs($other->vector[$op]-$frec);
				$taken +=$other->vector[$op];
			}else{
				$dif+=$frec;
			}
		}
		$dif += $other->size - $taken;
		return 100*(1-($dif/($this->size+$other->size)));
	}
}

class vpl_similarity_kr{
	static function similarity(&$simil,$level){
		//Calculate al hashes and store
		$hashes;
	}
}


/**
 * Utility class to get list of preprocessed files
 *
 */
class vpl_similarity{
	static public function scan_activity(&$simil, $vpl, $filesselected=array()){
		global $CFG;
		$vpl->require_capability(VPL_SIMILARITY_CAPABILITY);
		$list = get_users_by_capability($vpl->get_context(), 'moodle/legacy:student', '', "u.lastname ASC", '', '', '', '', false);
		if($list == null){
			return;
		}
		$submissions = $vpl->all_last_user_submission();
		foreach ($list as $userinfo) {
			if(isset($submissions[$userinfo->id])){
				$origin = '';
				$subinstance = $submissions[$userinfo->id];
				$submission = new mod_vpl_submission($vpl,$subinstance);
				$subf = $submission->get_submitted_fgm();
				$filelist = $subf->getFileList();
				foreach($filelist as $filename){
					if(count($filesselected)>0 && !isset($filesselected[$filename])){
						continue;
					}
					$sim = vpl_similarity_factory::get($filename);
					if($sim){
						$data = $subf->getFileData($filename);
						$from =new vpl_file_from_activity($filename,$vpl,$userinfo->id);
						$sim->init($data,$from);
						if($sim->size>10){
							$simil[]=$sim;
						}
					}
				}
			}
		}
	}
	
	static public function scan_zip(&$simil, $zipfile, $vpl, $filesselected=array()){
		global $CFG;
		$ext = strtoupper(pathinfo($zipfile,PATHINFO_EXTENSION));
		if($ext != 'ZIP'){
			print_error('nozipfile');
		}
		//TODO change to new zip class when available
		//Get class to handle zip
		require_once($CFG->libdir . '/filelib.php');
		require_once($CFG->libdir . '/pclzip/pclzip.lib.php');
		$course = $vpl->get_course();
		$dirfilename=tempnam($CFG->dataroot . '/temp/'  , 'vpl_dirextract' );
		$destdir=$dirfilename.'.dir';
		mkdir($destdir,0700);
		unlink($dirfilename);
		$zipfilename=$CFG->dataroot.'/'.$course->id.'/'.$zipfile;
		$zip = new PclZip($zipfilename);
		$listfiles = $zip->extract(PCLZIP_OPT_PATH, $destdir);
		$toremove=strlen($destdir);
		foreach($listfiles as $file){
			if(!$file['folder']){
				//TODO search userifo
				$filename=$file['filename'];
				if(count($filesselected)>0 && !isset($filesselected[basename($filename)])){
					continue;
				}
				$sim = vpl_similarity_factory::get($filename);
				if($sim){
					$data = file_get_contents($filename);
					//TODO locate userid
					$from =new vpl_file_from_zipfile(substr($filename,$toremove));
					$sim->init($data,$from);
					if($sim->size>10){
						$simil[]=$sim;
					}
				}
			}
		}
		//remove temp dir
		vpl_delete_dir($destdir);	
	}
	static function get_dir_filelist($basepath,$dir){
        $filelist = array();
        $dirlist = array();
        if($dir > ''){
        	$basedir = $basepath.'/'.$dir;
        }else{
        	$basedir = $basepath;
        }
        if($dd = opendir($basedir)){
			while ($file=readdir($dd)) {
				if ($file[0] == '.'){
					continue;
				}
				$filepath=$dir.'/'.$file;
				if(is_file($basedir."/".$filepath)){
					$filelist[] = $filepath;
				}elseif(is_dir($basedir."/".$filepath)){
					$dirlist[] = $filepath;
				}
			}
			closedir($dd);
        }
        foreach($dirlist as $adir){
        	$otherlist = self::get_dir_filelist($basepath,$adir);
        	$filelist = array_merge($filelist,$otherlist);
        }
        return $filelist;
	}
	static public function scan_directory(&$simil, $dir, $filesselected=array()){
		$filelist = self::get_dir_filelist($dir,'');
		foreach($filelist as $file){
			$base = basename($file);
			if(count($filesselected)>0 && !isset($filesselected[$base])){
				continue;
			}
			$sim = vpl_similarity_factory::get($dir.'/'.$file);
			if($sim){
				$data = file_get_contents($file);
				//TODO locate userid
				$from =new vpl_file_from_dir($file);
				$sim->init($data,$from);
				if($sim->size>10){
					$simil[]=$sim;
				}
			}
		}
	}
}

?>